@@ -5,10 +5,10 @@ by example_import.py
 
 
 def printMessage():
-    print('Running code from another file!')
+    getLogger().info('Running code from another file!')
 
 
-# Copyright (C) 2020, 2021 TestaLab
+# Copyright (C) 2020-2021 ImSwitch developers
 # This file is part of ImSwitch.
 #
 # ImSwitch is free software: you can redistribute it and/or modify
