@@ -3,8 +3,10 @@ from .AlignXYController import AlignXYController
 from .AlignmentLineController import AlignmentLineController
 from .BeadRecController import BeadRecController
 from .ConsoleController import ConsoleController
+from .EtSTEDController import EtSTEDController
 from .FFTController import FFTController
 from .FocusLockController import FocusLockController
+from .AufofocusController import AutofocusController
 from .ImageController import ImageController
 from .LaserController import LaserController
 from .PositionerController import PositionerController
@@ -14,3 +16,5 @@ from .ScanController import ScanController
 from .SettingsController import SettingsController
 from .ULensesController import ULensesController
 from .ViewController import ViewController
+from .TilingController import TilingController
+from .WatcherController import WatcherController
