@@ -3,3 +3,6 @@ from .SetupInfo import DeviceInfo, DetectorInfo, LaserInfo, PositionerInfo, Scan
 from .errors import *
 from .managers import *
 from .signaldesigners import SignalDesignerFactory
+import sys
+
+sys.modules['visa'] = 'pyvisa'
\ No newline at end of file
