@@ -1,7 +1,3 @@
-"""
-Created on Tue Apr  7 16:46:33 2020
-
-"""
 import operator
 import traceback
 import warnings
@@ -30,14 +26,27 @@ class NidaqManager(SignalInterface):
 
         self.__setupInfo = setupInfo
         self.tasks = {}
+        self.doTaskWaiter = None
+        self.aoTaskWaiter = None
+        self.timerTaskWaiter = None
         self.busy = False
-        self.__timerCounterChannel = setupInfo.nidaq.timerCounterChannel
+        self.__timerCounterChannel = setupInfo.nidaq.getTimerCounterChannel()
         self.__startTrigger = setupInfo.nidaq.startTrigger
 
+    def __del__(self):
+        for taskWaiter in [self.doTaskWaiter, self.aoTaskWaiter, self.timerTaskWaiter]:
+            if taskWaiter is not None:
+                taskWaiter.quit()
+                taskWaiter.wait()
+        if hasattr(super(), '__del__'):
+            super().__del__()
+
     def __makeSortedTargets(self, sortingKey):
         targetPairs = []
         for targetId, targetInfo in self.__setupInfo.getAllDevices().items():
             value = getattr(targetInfo, sortingKey)
+            if callable(value):
+                value = value()
             if value is not None:
                 pair = [targetId, value]
                 targetPairs.append(pair)
@@ -52,7 +61,7 @@ class NidaqManager(SignalInterface):
         channels = np.atleast_1d(channels)
 
         for channel in channels:
-            aotask.ao_channels.add_ao_voltage_chan('Dev1/ao%s' % channel,
+            aotask.ao_channels.add_ao_voltage_chan(channel,
                                                    min_val=min_val,
                                                    max_val=max_val)
         aotask.timing.cfg_samp_clk_timing(source=source,
@@ -71,7 +80,7 @@ class NidaqManager(SignalInterface):
         lines = np.atleast_1d(lines)
 
         for line in lines:
-            dotask.do_channels.add_do_chan('Dev1/port0/line%s' % line)
+            dotask.do_channels.add_do_chan(line)
         dotask.timing.cfg_samp_clk_timing(source=source, rate=rate,
                                           sample_mode=acquisitionType,
                                           samps_per_chan=sampsInScan)
@@ -84,7 +93,7 @@ class NidaqManager(SignalInterface):
         """ Simplified function to create a counter input task """
         citask = nidaqmx.Task(name)
         citaskchannel = citask.ci_channels.add_ci_count_edges_chan(
-            'Dev1/ctr%s' % channel,
+            channel,
             initial_count=0,
             edge=nidaqmx.constants.Edge.RISING,
             count_direction=nidaqmx.constants.CountDirection.COUNT_UP
@@ -112,7 +121,7 @@ class NidaqManager(SignalInterface):
                            reference_trigger='ai/StartTrigger'):
         cotask = nidaqmx.Task(name)
         self.cotaskchannel = cotask.co_channels.add_co_pulse_chan_freq(
-            'Dev1/ctr%s' % channel, freq=rate, units=nidaqmx.constants.FrequencyUnits.HZ
+            channel, freq=rate, units=nidaqmx.constants.FrequencyUnits.HZ
         )
         # cotask.timing.cfg_implicit_timing(sample_mode=nidaqmx.constants.AcquisitionType.CONTINUOUS)
         cotask.timing.cfg_implicit_timing(sample_mode=nidaqmx.constants.AcquisitionType.FINITE,
@@ -130,7 +139,7 @@ class NidaqManager(SignalInterface):
         """ Simplified function to create an analog input task """
         aitask = nidaqmx.Task(name)
         for channel in channels:
-            aitask.ai_channels.add_ai_voltage_chan('Dev1/ai%s' % channel)
+            aitask.ai_channels.add_ai_voltage_chan(channel)
         aitask.timing.cfg_samp_clk_timing(source=source,
                                           rate=rate,
                                           sample_mode=acquisitionType,
@@ -145,8 +154,8 @@ class NidaqManager(SignalInterface):
         # citask = nidaqmx.CounterInputTask(name)
         citask = nidaqmx.Task(name)
         # for channel in channels:
-        citask.create_channel_count_edges('Dev1/ctr' % channel, init=0)
-        citask.set_terminal_count_edges('Dev1/ctr' % channel, "PFI0")
+        citask.create_channel_count_edges(channel, init=0)
+        citask.set_terminal_count_edges(channel, "PFI0")
 
         citask.configure_timing_sample_clock(source=source,
                                              sample_mode=acquisitionType,
@@ -177,6 +186,7 @@ class NidaqManager(SignalInterface):
 
     def readInputTask(self, taskName, samples=0, timeout=False):
         if not timeout:
+            #self.__logger.debug(f'Read {samples} samples from {taskName}')
             return self.tasks[taskName].read(samples)
         else:
             return self.tasks[taskName].read(samples, timeout)
@@ -184,7 +194,7 @@ class NidaqManager(SignalInterface):
     def setDigital(self, target, enable):
         """ Function to set the digital line to a specific target
         to either "high" or "low" voltage """
-        line = self.__setupInfo.getDevice(target).digitalLine
+        line = self.__setupInfo.getDevice(target).getDigitalLine()
         if line is None:
             raise NidaqManagerError('Target has no digital output assigned to it')
         else:
@@ -220,7 +230,7 @@ class NidaqManager(SignalInterface):
     def setAnalog(self, target, voltage, min_val=-1, max_val=1):
         """ Function to set the analog channel to a specific target
         to a certain voltage """
-        channel = self.__setupInfo.getDevice(target).analogChannel
+        channel = self.__setupInfo.getDevice(target).getAnalogChannel()
         if channel is None:
             raise NidaqManagerError('Target has no analog output assigned to it')
         else:
@@ -266,19 +276,20 @@ class NidaqManager(SignalInterface):
                 stageDic = signalDic['scanSignalsDict']
                 ttlDic = signalDic['TTLCycleSignalsDict']
 
-                AOTargetChanPairs = self.__makeSortedTargets('analogChannel')
+                AOTargetChanPairs = self.__makeSortedTargets('getAnalogChannel')
                 AOdevices = []
                 AOsignals = []
                 AOchannels = []
 
                 for device, channel in AOTargetChanPairs:
+                    #self.__logger.debug(f'Device {device}, channel {channel} is part of scan')
                     if device not in stageDic:
                         continue
                     AOdevices.append(device)
                     AOsignals.append(stageDic[device])
                     AOchannels.append(channel)
 
-                DOTargetChanPairs = self.__makeSortedTargets('digitalLine')
+                DOTargetChanPairs = self.__makeSortedTargets('getDigitalLine')
                 DOdevices = []
                 DOsignals = []
                 DOlines = []
@@ -299,12 +310,13 @@ class NidaqManager(SignalInterface):
                 if self.__timerCounterChannel is not None:
                     self.timerTaskWaiter = WaitThread()
                     # create timer counter output task, to control the acquisition timing (1 MHz)
-                    sampsInScan = int(
-                        len(AOsignals[0] if len(AOsignals) > 0 else DOsignals[0]) * 10
+                    detSampsInScan = int(
+                        len(AOsignals[0] if len(AOsignals) > 0 else DOsignals[0]) * (1e6/100e3)
                     )
+                    #self.__logger.debug(f'Total detection samples in scan: {detSampsInScan}')
                     self.timerTask = self.__createChanCOTask(
                         'TimerTask', channel=self.__timerCounterChannel, rate=1e6,
-                        sampsInScan=sampsInScan, starttrig=self.__startTrigger,
+                        sampsInScan=detSampsInScan, starttrig=self.__startTrigger,
                         reference_trigger='ao/StartTrigger'
                     )
                     self.timerTaskWaiter.connect(self.timerTask)
@@ -316,11 +328,12 @@ class NidaqManager(SignalInterface):
                 scanclock = r'100kHzTimebase'
                 clockDO = scanclock
                 if len(AOsignals) > 0:
-                    sampsInScan = len(AOsignals[0])
+                    scanSampsInScan = len(AOsignals[0])
+                    self.__logger.debug(f'Total scan samples in scan: {scanSampsInScan}')
                     self.aoTask = self.__createChanAOTask('ScanAOTask', AOchannels,
                                                           acquisitionTypeFinite, scanclock,
                                                           100000, min_val=-10, max_val=10,
-                                                          sampsInScan=sampsInScan,
+                                                          sampsInScan=scanSampsInScan,
                                                           starttrig=False)
                     self.tasks['ao'] = self.aoTask
 
@@ -334,10 +347,10 @@ class NidaqManager(SignalInterface):
                     )
                     clockDO = r'ao/SampleClock'
                 if len(DOsignals) > 0:
-                    sampsInScan = len(DOsignals[0])
+                    scanSampsInScan = len(DOsignals[0])
                     self.doTask = self.__createLineDOTask('ScanDOTask', DOlines,
                                                           acquisitionTypeFinite, clockDO,
-                                                          100000, sampsInScan=sampsInScan,
+                                                          100000, sampsInScan=scanSampsInScan,
                                                           starttrig=self.__startTrigger,
                                                           reference_trigger='ao/StartTrigger')
                     self.tasks['do'] = self.doTask
@@ -435,7 +448,7 @@ class NidaqManagerError(Exception):
         self.message = message
 
 
-# Copyright (C) 2020, 2021 TestaLab
+# Copyright (C) 2020-2021 ImSwitch developers
 # This file is part of ImSwitch.
 #
 # ImSwitch is free software: you can redistribute it and/or modify
