@@ -10,6 +10,10 @@ class LaserWidget(Widget):
 
     sigEnableChanged = QtCore.Signal(str, bool)  # (laserName, enabled)
     sigValueChanged = QtCore.Signal(str, float)  # (laserName, value)
+    
+    sigModEnabledChanged = QtCore.Signal(str, bool) # (laserName, modulationEnabled)
+    sigFreqChanged = QtCore.Signal(str, int)        # (laserName, frequency)
+    sigDutyCycleChanged = QtCore.Signal(str, int)   # (laserName, dutyCycle)
 
     sigPresetSelected = QtCore.Signal(str)  # (presetName)
     sigLoadPresetClicked = QtCore.Signal()
@@ -80,14 +84,17 @@ class LaserWidget(Widget):
         self.layout.addLayout(self.presetsBox, 1, 0)
 
     def addLaser(self, laserName, valueUnits, valueDecimals, wavelength, valueRange=None,
-                 valueRangeStep=1):
+                 valueRangeStep=1, frequencyRange=(0, 0, 0)):
         """ Adds a laser module widget. valueRange is either a tuple
-        (min, max), or None (if the laser can only be turned on/off). """
+        (min, max), or None (if the laser can only be turned on/off).
+        frequencyRange is either a tuple (min, max, initVal)
+        or (0, 0, 0) (if the laser is not modulated in frequency)"""
 
         control = LaserModule(
             valueUnits=valueUnits, valueDecimals=valueDecimals, valueRange=valueRange,
             tickInterval=5, singleStep=valueRangeStep,
-            initialPower=valueRange[0] if valueRange is not None else 0
+            initialPower=valueRange[0] if valueRange is not None else 0,
+            frequencyRange=frequencyRange
         )
         control.sigEnableChanged.connect(
             lambda enabled: self.sigEnableChanged.emit(laserName, enabled)
@@ -96,6 +103,17 @@ class LaserWidget(Widget):
             lambda value: self.sigValueChanged.emit(laserName, value)
         )
 
+        if all(num > 0 for num in frequencyRange):
+            control.sigModEnabledChanged.connect(
+                lambda enabled: self.sigModEnabledChanged.emit(laserName, enabled)
+            )
+            control.sigFreqChanged.connect(
+                lambda frequency: self.sigFreqChanged.emit(laserName, frequency)
+            )
+            control.sigDutyCycleChanged.connect(
+                lambda dutyCycle: self.sigDutyCycleChanged.emit(laserName, dutyCycle)
+            )
+
         nameLabel = QtWidgets.QLabel(laserName)
         color = colorutils.wavelengthToHex(wavelength)
         nameLabel.setStyleSheet(
@@ -138,6 +156,14 @@ class LaserWidget(Widget):
         """ Sets the value of the specified laser, in the units that the laser
         uses. """
         self.laserModules[laserName].setValue(value)
+    
+    def setModulationFrequency(self, laserName, value):
+        """ Sets the modulation frequency of the specified laser. """
+        self.laserModules[laserName].setModulationFrequency(value)
+
+    def setModulationDutyCycle(self, laserName, value):
+        """ Sets the modulation duty cycle of the specified laser. """
+        self.laserModules[laserName].setModulationDutyCycle(value)
 
     def getCurrentPreset(self):
         """ Returns the name of the currently selected preset. """
@@ -207,15 +233,20 @@ class LaserModule(QtWidgets.QWidget):
     sigEnableChanged = QtCore.Signal(bool)  # (enabled)
     sigValueChanged = QtCore.Signal(float)  # (value)
 
+    sigModEnabledChanged = QtCore.Signal(bool) # (modulation enabled)
+    sigFreqChanged = QtCore.Signal(int)        # (frequency)
+    sigDutyCycleChanged = QtCore.Signal(int)   # (duty cycle)
+
     def __init__(self, valueUnits, valueDecimals, valueRange, tickInterval, singleStep,
-                 initialPower, *args, **kwargs):
+                 initialPower, frequencyRange, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.valueDecimals = valueDecimals
 
         isBinary = valueRange is None
+        isModulated = all(num > 0 for num in frequencyRange)
 
         # Graphical elements
-        self.setPointLabel = QtWidgets.QLabel('Setpoint')
+        self.setPointLabel = QtWidgets.QLabel(f'Setpoint [{valueUnits}]')
         self.setPointLabel.setAlignment(QtCore.Qt.AlignCenter)
         self.setPointEdit = QtWidgets.QLineEdit(str(initialPower))
         self.setPointEdit.setFixedWidth(50)
@@ -247,13 +278,66 @@ class LaserModule(QtWidgets.QWidget):
         powerFrame.setFrameStyle(QtWidgets.QFrame.Panel | QtWidgets.QFrame.Plain)
         powerFrame.setLayout(self.powerGrid)
 
-        self.powerGrid.addWidget(self.setPointLabel, 0, 0, 1, 2)
-        self.powerGrid.addWidget(self.setPointEdit, 1, 0)
-        self.powerGrid.addWidget(QtWidgets.QLabel(valueUnits), 1, 1)
+        self.powerGrid.addWidget(self.setPointLabel, 0, 0, 2, 1)
+        self.powerGrid.addWidget(self.setPointEdit, 0, 1, 2, 1)
         self.powerGrid.addWidget(self.minpower, 0, 2, 2, 1)
-        self.powerGrid.addWidget(self.slider, 0, 3, 2, 4)
-        self.powerGrid.addWidget(self.maxpower, 0, 8, 2, 1)
+        self.powerGrid.addWidget(self.slider, 0, 3, 2, 1)
+        self.powerGrid.addWidget(self.maxpower, 0, 4, 2, 1)
+        
+        if isModulated:
+            freqRangeMin, freqRangeMax, initialFrequency = frequencyRange
+            # laser modulation widgets
+            # enable button
+            self.modulationEnable = guitools.BetterPushButton("ON")
+            self.modulationEnable.setSizePolicy(QtWidgets.QSizePolicy.Minimum,
+                                            QtWidgets.QSizePolicy.Expanding)
+            self.modulationEnable.setCheckable(True)
 
+            # frequency slider
+            self.modulationFrequencyLabel = QtWidgets.QLabel("Frequency [Hz]")
+            self.modulationFrequencyLabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
+            self.modulationFrequencyEdit = QtWidgets.QLineEdit(str(initialFrequency))
+            self.modulationFrequencyEdit.setFixedWidth(50)
+            self.modulationFrequencyEdit.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
+            self.modulationFrequencyMinLabel = QtWidgets.QLabel(str(freqRangeMin))
+            self.modulationFrequencyMaxLabel = QtWidgets.QLabel(str(freqRangeMax))
+            self.modulationFrequencySlider = guitools.BetterSlider(QtCore.Qt.Horizontal)
+            self.modulationFrequencySlider.setRange(freqRangeMin, freqRangeMax)
+            self.modulationFrequencySlider.setValue(initialFrequency)
+
+            # duty cycle slider
+            self.modulationDutyCycleLabel = QtWidgets.QLabel("Duty cycle [%]")
+            self.modulationDutyCycleLabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
+            self.modulationDutyCycleEdit = QtWidgets.QLineEdit(str(50))
+            self.modulationDutyCycleEdit.setFixedWidth(50)
+            self.modulationDutyCycleEdit.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
+            self.modulationDutyCycleMinLabel = QtWidgets.QLabel(str(1))
+            self.modulationDutyCycleMaxLabel = QtWidgets.QLabel(str(99))
+            self.modulationDutyCycleMinLabel.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
+            self.modulationDutyCycleMaxLabel.setAlignment(QtCore.Qt.AlignLeft | QtCore.Qt.AlignVCenter)
+            self.modulationDutyCycleSlider = guitools.BetterSlider(QtCore.Qt.Horizontal)
+            self.modulationDutyCycleSlider.setRange(1, 99)
+            self.modulationDutyCycleSlider.setValue(50)
+
+            self.modulationGroup = QtWidgets.QGroupBox("Frequency modulation")
+            self.modulationLayout = QtWidgets.QGridLayout()
+
+            self.modulationLayout.addWidget(self.modulationFrequencyLabel, 0, 0)
+            self.modulationLayout.addWidget(self.modulationFrequencyEdit, 0, 1)
+            self.modulationLayout.addWidget(self.modulationFrequencyMinLabel, 0, 2)
+            self.modulationLayout.addWidget(self.modulationFrequencySlider, 0, 3)
+            self.modulationLayout.addWidget(self.modulationFrequencyMaxLabel, 0, 4)
+
+            self.modulationLayout.addWidget(self.modulationDutyCycleLabel, 1, 0)
+            self.modulationLayout.addWidget(self.modulationDutyCycleEdit, 1, 1)
+            self.modulationLayout.addWidget(self.modulationDutyCycleMinLabel, 1, 2)
+            self.modulationLayout.addWidget(self.modulationDutyCycleSlider, 1, 3)
+            self.modulationLayout.addWidget(self.modulationDutyCycleMaxLabel, 1, 4)
+            self.modulationLayout.addWidget(self.modulationEnable, 0, 5, 2, 1)
+            self.modulationGroup.setLayout(self.modulationLayout)
+
+            self.powerGrid.addWidget(self.modulationGroup, 2, 0, 1, 5)
+                
         self.enableButton = guitools.BetterPushButton('ON')
         self.enableButton.setSizePolicy(QtWidgets.QSizePolicy.Minimum,
                                         QtWidgets.QSizePolicy.Expanding)
@@ -280,6 +364,21 @@ class LaserModule(QtWidgets.QWidget):
             lambda: self.sigValueChanged.emit(self.getValue())
         )
 
+        if isModulated:
+            self.modulationEnable.toggled.connect(self.sigModEnabledChanged)
+            self.modulationFrequencySlider.valueChanged.connect(
+                lambda value: self.sigFreqChanged.emit(value)
+            )
+            self.modulationFrequencyEdit.returnPressed.connect(
+                lambda: self.sigFreqChanged.emit(self.getFrequency())
+            )
+            self.modulationDutyCycleSlider.valueChanged.connect(
+                lambda value: self.sigDutyCycleChanged.emit(value)
+            )
+            self.modulationDutyCycleEdit.returnPressed.connect(
+                lambda: self.sigDutyCycleChanged.emit(self.getDutyCycle())
+            )
+
     def isActive(self):
         """ Returns whether the laser is powered on. """
         return self.enableButton.isChecked()
@@ -288,6 +387,16 @@ class LaserModule(QtWidgets.QWidget):
         """ Returns the value of the laser, in the units that the laser
         uses. """
         return float(self.setPointEdit.text())
+    
+    def getFrequency(self):
+        """ Returns the selected frequency of the laser.
+        """
+        return int(self.modulationFrequencyEdit.text())
+    
+    def getDutyCycle(self):
+        """ Returns the selected duty cycle of the laser.
+        """
+        return int(self.modulationDutyCycleEdit.text())
 
     def setActive(self, active):
         """ Sets whether the laser is powered on. """
@@ -307,9 +416,19 @@ class LaserModule(QtWidgets.QWidget):
         """ Sets the value of the laser, in the units that the laser uses. """
         self.setPointEdit.setText(f'%.{self.valueDecimals}f' % value)
         self.slider.setValue(value)
+    
+    def setModulationFrequency(self, value):
+        """ Sets the laser modulation frequency. """
+        self.modulationFrequencyEdit.setText(f"{value}")
+        self.modulationFrequencySlider.setValue(value)
+    
+    def setModulationDutyCycle(self, value):
+        """ Sets the laser modulation duty cycle. """
+        self.modulationDutyCycleEdit.setText(f"{value}")
+        self.modulationDutyCycleSlider.setValue(value)
 
 
-# Copyright (C) 2017 Federico Barabas 2020, 2021 TestaLab
+# Copyright (C) 2017 Federico Barabas 2020-2021 ImSwitch developers
 # This file is part of Tormenta and ImSwitch.
 #
 # Tormenta and ImSwitch are free software: you can redistribute it and/or modify
