@@ -3,8 +3,10 @@ from .AlignXYWidget import AlignXYWidget
 from .AlignmentLineWidget import AlignmentLineWidget
 from .BeadRecWidget import BeadRecWidget
 from .ConsoleWidget import ConsoleWidget
+from .EtSTEDWidget import EtSTEDWidget
 from .FFTWidget import FFTWidget
 from .FocusLockWidget import FocusLockWidget
+from .AutofocusWidget import AutofocusWidget
 from .ImageWidget import ImageWidget
 from .LaserWidget import LaserWidget
 from .PositionerWidget import PositionerWidget
@@ -13,5 +15,7 @@ from .SLMWidget import SLMWidget
 from .ScanWidget import ScanWidget
 from .SettingsWidget import SettingsWidget
 from .ULensesWidget import ULensesWidget
+from .WatcherWidget import WatcherWidget
 from .ViewWidget import ViewWidget
-from .basewidgets import WidgetFactory
+from .TilingWidget import TilingWidget
+from .basewidgets import WidgetFactory
\ No newline at end of file
