@@ -21,8 +21,12 @@ class CommunicationChannel(SignalInterface):
 
     sigAddToMultiData = Signal(str, str)  # (path, datasetName)
 
+    sigReconstruct = Signal(object, bool)
 
-# Copyright (C) 2020, 2021 TestaLab
+    sigExecutionFinished = Signal(object)
+
+
+# Copyright (C) 2020-2021 ImSwitch developers
 # This file is part of ImSwitch.
 #
 # ImSwitch is free software: you can redistribute it and/or modify
